/* 
**  mod_iowa.c -- Apache sample iowa module
**  [Autogenerated via ``apxs -n iowa -g'']
**
**  To play with this sample module, first compile it into a
**  DSO file and install it into Apache's libexec directory 
**  by running:
**
**    $ apxs -c -i mod_iowa.c
**
**  Then activate it in Apache's httpd.conf file, for instance
**  for the URL /iowa, as follows:
**
**    #   httpd.conf
**    LoadModule iowa_module libexec/mod_iowa.so
**    <Location /iowa>
**    SetHandler iowa
**    </Location>
**
**  Then after restarting Apache via
**
**    $ apachectl restart
**
**  you immediately can request the URL /%NAME and watch for the
**  output of this module. This can be achieved for instance via:
**
**    $ lynx -mime_header http://localhost/iowa 
**
**  The output should be similar to the following one:
**
**    HTTP/1.1 200 OK
**    Date: Tue, 31 Mar 1998 14:42:22 GMT
**    Server: Apache/1.3.4 (Unix)
**    Connection: close
**    Content-Type: text/html
**  
**    The sample page from mod_iowa.c
*/ 

#include "httpd.h"
#include "http_config.h"
#include "http_protocol.h"
#include "ap_config.h"

#include <sys/socket.h>
#include <sys/un.h>

static int iowa_handler(request_rec *r)
{
    r->content_type = "text/html";      
    ap_send_http_header(r);
    if (!r->header_only)
	{
		char *uri;
		char *app;
		char urilen;
		char applen;
		struct sockaddr_un addr;
		int s = socket(AF_UNIX, SOCK_STREAM, 0);
		int len;
		int err;			
		char buf[HUGE_STRING_LEN];
		
		uri = r->uri;
		urilen = strlen(uri);

		app = strstr(uri, "iowa");
		app = strchr(app, '/');
		app++;
		applen = strcspn(app, "/");
	
		strcpy(buf, "/tmp/iowa_");
		strcpy(buf+10, app);
		buf[applen+10] = 0;
	
		addr.sun_family = AF_UNIX;
		strcpy(addr.sun_path, buf);
		len = strlen(addr.sun_path) + sizeof(addr.sun_family);
		err = connect(s, &addr, len);
		
		send(s, &urilen, 1, 0);
		send(s, uri, urilen, 0);
	

		if(r->args)
			send(s, r->args, strlen(r->args), 0);
		else
		{
			ap_setup_client_block(r, REQUEST_CHUNKED_ERROR);
			while(len = ap_get_client_block(r, buf, HUGE_STRING_LEN))
				send(s, buf, len, 0);
		}
		shutdown(s, 1);

		while((len = recv(s, buf, HUGE_STRING_LEN, 0)) != 0)
		{
			if (len == -1)
			{
				if (errno == EAGAIN)
					continue;
				else
					break;
			}
			
			ap_rwrite(buf, len, r);
		}
 	}
    return OK;
}

/* Dispatch list of content handlers */
static const handler_rec iowa_handlers[] = { 
    { "iowa", iowa_handler }, 
    { NULL, NULL }
};

/* Dispatch list for API hooks */
module MODULE_VAR_EXPORT iowa_module = {
    STANDARD_MODULE_STUFF, 
    NULL,                  /* module initializer                  */
    NULL,                  /* create per-dir    config structures */
    NULL,                  /* merge  per-dir    config structures */
    NULL,                  /* create per-server config structures */
    NULL,                  /* merge  per-server config structures */
    NULL,                  /* table of config file commands       */
    iowa_handlers,       /* [#8] MIME-typed-dispatched handlers */
    NULL,                  /* [#1] URI to filename translation    */
    NULL,                  /* [#4] validate user id from request  */
    NULL,                  /* [#5] check if the user is ok _here_ */
    NULL,                  /* [#3] check access by host address   */
    NULL,                  /* [#6] determine MIME type            */
    NULL,                  /* [#7] pre-run fixups                 */
    NULL,                  /* [#9] log a transaction              */
    NULL,                  /* [#2] header parser                  */
    NULL,                  /* child_init                          */
    NULL,                  /* child_exit                          */
    NULL                   /* [#0] post read-request              */
#ifdef EAPI
   ,NULL,                  /* EAPI: add_module                    */
    NULL,                  /* EAPI: remove_module                 */
    NULL,                  /* EAPI: rewrite_command               */
    NULL                   /* EAPI: new_connection                */
#endif
};

